import java.util.*;
import acm.program.*;
import acm.util.*;

public class SortTiming extends ConsoleProgram {
	/* How many iterations to run the sorting algorithm. */
	private static final int NUM_ITERATIONS = 100;
	
	/* How many elements are in the array to sort. */
	private static final int NUM_ELEMENTS = 5000;
	
	public void run() {
		/* Who cares if we can sort if we can't see the results! */
		setFont("DejaVuSerif-BOLD-24");
		
		/* See how long it takes to sort these values. */
		timeSort(NUM_ELEMENTS);
	}
	
	/**
	 * Runs many iterations of a sorting algorithm on an array of some particular
	 * size, reporting how long it took for all the sorts to complete.
	 * 
	 * @param arraySize The size of the array.
	 */
	private void timeSort(int arraySize) {	
		int[] arr = new int[arraySize];
		
		/* Monitor the start time. */
		Date now = new Date();
		
		/* Generate and sort many arrays of values. */
		for (int i = 0; i < NUM_ITERATIONS; i++) {
			fillArray(arr);
			selectionSort(arr);
			
			/* Or, try this! */
			// Arrays.sort(arr);
		}
			
		/* Determine the total runtime. */
		long elapsed = new Date().getTime() - now.getTime();
		println("Elapsed time: " + elapsed + "ms");
	}
	
	/**
	 * Fills an array with randomly-generated integers.
	 * 
	 * @param array The array to fill.
	 */
	private void fillArray(int[] array) {
		RandomGenerator rgen = RandomGenerator.getInstance();
		for (int i = 0; i < array.length; i++) {
			array[i] = rgen.nextInt();
		}
	}
	
	/**
	 * Runs selection sort on the given array, putting it into sorted order.
	 * 
	 * @param array The array to sort.
	 */
	private void selectionSort(int[] array) {
	    for (int index = 0; index < array.length; index++) {
	        int smallestIndex = indexOfSmallest(array, index);
	        swap(array, index, smallestIndex);
	    }
	}

	/**
	 * Finds the index of the smallest value in array at or after the indicated
	 * starting point.
	 * 
	 * @param array The array to search.
	 * @param startPoint The starting point.
	 * @return The index of the smallest value after that point.
	 */
	private int indexOfSmallest(int[] array, int startPoint) {
	    int smallestIndex = startPoint;
	    for (int i = startPoint + 1; i < array.length; i++) {
	        if (array[i] < array[smallestIndex]) {
	            smallestIndex = i;
	        }
	    }
	    return smallestIndex;
	}

	/**
	 * Swaps two elements of an array.
	 * 
	 * @param array The array.
	 * @param i The index of the first element.
	 * @param j The index of the second element.
	 */
	private void swap(int[] array, int i, int j) {
	    int temp = array[i];
	    array[i] = array[j];
	    array[j] = temp;
	}
}
